{#
# This file is part of kwalitee
# Copyright (C) 2014, 2015 CERN.
#
# kwalitee is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# kwalitee is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with kwalitee; if not, write to the Free Software Foundation,
# Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
#
# In applying this licence, CERN does not waive the privileges and immunities
# granted to it by virtue of its status as an Intergovernmental Organization
# or submit itself to any jurisdiction.
#}

{%- extends 'page.html' -%}
{%- import "helpers.html" as helpers -%}

{%- block body -%}
<div class="row">
  <div class="col-lg-12">
    <h2>
      {{ helpers.repository(account, repository) -}}
      /<a href="{{ url_for("branch",
                           account=account.name,
                           repository=repository.name,
                           branch=branches[0].name) }}">
        {{- branches[0].name -}}
      </a>
    </h2>
    <ul>
  {%- for branch in branches %}
      <li>
        <a href="{{ url_for("branch_status",
                            account=account.name,
                            repository=repository.name,
                            branch=branch.name,
                            sha=branch.commit.sha) }}">
          {{ branch.commit.sha | truncate(7, True, end="")}}
        </a>
    {%- if branch.errors %}
        {{ "{0} errors.".format(branch.errors) }}
    {%- else %}
        Everything is OK.
    {%- endif %}
        (<a href="{{ branch.url }}">github</a>)
      </li>
  {%- endfor %}
  </div>
</div>
{%- endblock body -%}
