{#
# This file is part of kwalitee
# Copyright (C) 2014, 2015 CERN.
#
# kwalitee is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# kwalitee is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with kwalitee; if not, write to the Free Software Foundation,
# Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
#
# In applying this licence, CERN does not waive the privileges and immunities
# granted to it by virtue of its status as an Intergovernmental Organization
# or submit itself to any jurisdiction.
#}

{%- extends 'page.html' -%}

{%- block body -%}
<div class="row">
  <div class="col-lg-12">
    <h2>Repositories of {{ account.name }}</h2>
    <ul>
  {%- for repository in repositories %}
      <li>
        <a href="{{ url_for("repository", account=account.name,
                                          repository=repository.name) }}">
          {{- repository.name -}}
        </a>
        (<a href="{{ config.get("GITHUB_REPO").format(account=account.name,
                                                      repository=repository.name) }}">
          github
        </a>)
      </li>
  {%- endfor %}
    </ul>
  </div>
</div>
{%- endblock body -%}
