{#
# This file is part of kwalitee
# Copyright (C) 2014, 2015 CERN.
#
# kwalitee is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# kwalitee is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with kwalitee; if not, write to the Free Software Foundation,
# Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
#
# In applying this licence, CERN does not waive the privileges and immunities
# granted to it by virtue of its status as an Intergovernmental Organization
# or submit itself to any jurisdiction.
#}

{%- macro repository(account, repository, sep="/") -%}
{# Build the repository title bar: account/repository.
 #
 # The account can be left empty but it's not recommended.
 #}
{%- if not account or account.id != repository.owner_id -%}
  {%- set account = repository.owner -%}
{%- endif -%}
<a href="{{ url_for("account", account=account.name) }}">
  {{- account.name -}}
</a>
{{- sep -}}
<a href="{{ url_for("repository",
                         account=account.name,
                         repository=repository.name ) }}">
  {{- repository.name -}}
</a>
{%- endmacro -%}

{%- macro files(content) -%}
{# Build the file errors view.
 #
 # if content is empty then everything is ok
 #}
  {%- if content %}
<h3>Files</h3>
    {%- for filename, data in content.items() -%}
      {%- if data["errors"] %}
<h4>{{ filename }}</h4>
<pre>
      {%- for error in data["errors"] -%}
  {{- error }}
{% endfor -%}
</pre>
      {%- else %}
        <!-- {{ filename}} was ok -->
      {%- endif %}
    {%- endfor %}
  {%- else %}
<p>Everything is OK.</p>
  {%- endif %}
{%- endmacro -%}
