{#
# This file is part of kwalitee
# Copyright (C) 2014, 2015 CERN.
#
# kwalitee is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# kwalitee is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with kwalitee; if not, write to the Free Software Foundation,
# Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
#
# In applying this licence, CERN does not waive the privileges and immunities
# granted to it by virtue of its status as an Intergovernmental Organization
# or submit itself to any jurisdiction.
#}

{%- extends 'page.html' -%}
{%- import "helpers.html" as helpers -%}

{%- block body -%}
{%- set content = commit.content -%}
<div class="row">
  <div class="col-lg-12">
    <h2>
      {{ helpers.repository(account, repository) -}}
      /<a href="{{ url_for("commit",
                           account=account.name,
                           repository=repository.name,
                           sha=commit.sha) }}">
        {{- commit.sha | truncate(7, True, end="") -}}
      </a>
      (<a href="{{ commit.url }}">github</a>)
    </h2>
  {%- if commit.is_pending() %}
    <p>
      Pending
    <p>
  {%- else %}
    {%- if content["message"] %}
    <pre>
        {%- for line in content["message"] -%}
        {{- line }}
{% endfor -%}
    </pre>
    {%- endif %}
    {{ helpers.files(content.get("files")) }}
  </div>
  {%- endif %}
</div>
{%- endblock body -%}
